//  This class encapsulates Performer Intersection detection.  This
//  can then be used to implement collision detection.

class dpfIntersectionNode {

public:
  dpfIntersectionNode(void);

  //  Peform intersection detection with the current segments on the current
  //  scenegraph.
  void doIntersection();

  //  Set the root of the scenegraph that will be used for intersection testing
  void setRoot(pfNode* rootNode);

  //  Enable the specified segment.  By default, all segments are disabled
  //  Retuns 0 if successful
  int enableSegment(int segId);

  //  Disable the specified segment.
  //  Retuns 0 if successful
  int disableSegment(int segId);

  //  Set the position of the starting point of the specified segment.  
  //  This should be in the coordinate system of the root node
  //  Retuns 0 if successful
  int setSegmentPos(int segId, float x, float y, float z);

  //  Set the position of the starting point of the specified segment,
  //  given a pfVec3.  This should be in the coordinate system of the root node
  int setSegmentPos(int segId, pfVec3 position); 

  //  Set the direction of the specified segment.  This should be in the
  //  coordinate system of the root node
  int setSegmentDir(int segId, float x, float y, float z);

  //  Set the direction of the starting point of the specified
  //  segment, given a pfVec3.  This should be in the coordinate
  //  system of the root node
  int setSegmentDir(int segId, pfVec3 direction); 

  //  Set the length of the specified segment
  int setSegmentLength(int segId, float len);

  //  Determine if the given segment was hit
  //  Returns -1 on error, 0 if no hit, 1 if a hit
  int segmentHit(int segId);
 
  //  Get the coordinate of the hit for the specified segment, transformed to
  //  the coordinate system of the root node.  Store the result in the
  //  parameter point.
  //  Retuns 0 if successful
  int getXFormPoint(int segId, pfVec3* point);

  //  Get the coordinate of the hit for the specified segment in local
  //  object coordinates.  Store the result in the parameter point.
  //  Retuns 0 if successful
  int getLocalPoint(int segId, pfVec3* point);

  // Get the path of the node that is hit by the specified segment
  //  Retuns 0 if successful
  pfPath* getHitPath(int segId);

  //  Convenience method to make an object invisible to Intersection detection
  //  Returns 0 on success
  int noIntersection(pfNode* node);

  //  Convenience method to make an object visible again
  //  Returns 0 on success
  int yesIntersection(pfNode* node);

  ~dpfIntersectionNode();

private:
  pfNode* root;

  pfHit** hits[PFIS_MAX_SEGS];

  int nHits;

  pfSegSet* segset;
};
